@@ -1,5 +1,5 @@
 #
-# $Id: Base64.pm,v 2.12 1998/11/13 22:04:00 aas Exp $
+# $Id: Base64.pm,v 2.19 2002/12/28 06:32:37 gisle Exp $
 
 package MIME::Base64;
 
@@ -48,7 +48,7 @@ silently ignored.  Characters occuring after a '=' padding character
 are never decoded.
 
 If the length of the string to decode (after ignoring
-non-base64 chars) is not a multiple of 4 or padding occurs too ealy,
+non-base64 chars) is not a multiple of 4 or padding occurs too early,
 then a warning is generated if perl is running under C<-w>.
 
 =back
@@ -101,9 +101,18 @@ or if you know you have enough memory
    local($/) = undef;  # slurp
    print encode_base64(<STDIN>);
 
+The same approach as a command line:
+
+   perl -MMIME::Base64 -0777 -ne 'print encode_base64($_)' <file
+
+Decoding does not need slurp mode if all the lines contains a multiple
+of 4 base64 chars:
+
+   perl -MMIME::Base64 -ne 'print decode_base64($_)' <file
+
 =head1 COPYRIGHT
 
-Copyright 1995-1998 Gisle Aas.
+Copyright 1995-1999, 2001 Gisle Aas.
 
 This library is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
@@ -126,7 +135,7 @@ require DynaLoader;
 @ISA = qw(Exporter DynaLoader);
 @EXPORT = qw(encode_base64 decode_base64);
 
-$VERSION = '2.09';
+$VERSION = '2.13';
 
 eval { bootstrap MIME::Base64 $VERSION; };
 if ($@) {
@@ -146,14 +155,14 @@ use integer;
 
 sub old_encode_base64 ($;$)
 {
-    my $res = "";
     my $eol = $_[1];
     $eol = "\n" unless defined $eol;
-    pos($_[0]) = 0;                          # ensure start at the beginning
-    while ($_[0] =~ /(.{1,45})/gs) {
-	$res .= substr(pack('u', $1), 1);
-	chop($res);
-    }
+
+    my $res = pack("u", $_[0]);
+    # Remove first character of each line, remove newlines
+    $res =~ s/^.//mg;
+    $res =~ s/\n//g;
+
     $res =~ tr|` -_|AA-Za-z0-9+/|;               # `# help emacs
     # fix padding at the end
     my $padding = (3 - length($_[0]) % 3) % 3;
@@ -162,7 +171,7 @@ sub old_encode_base64 ($;$)
     if (length $eol) {
 	$res =~ s/(.{1,76})/$1$eol/g;
     }
-    $res;
+    return $res;
 }
 
 
@@ -171,8 +180,6 @@ sub old_decode_base64 ($)
     local($^W) = 0; # unpack("u",...) gives bogus warning in 5.00[123]
 
     my $str = shift;
-    my $res = "";
-
     $str =~ tr|A-Za-z0-9+=/||cd;            # remove non-base64 chars
     if (length($str) % 4) {
 	require Carp;
@@ -180,11 +187,23 @@ sub old_decode_base64 ($)
     }
     $str =~ s/=+$//;                        # remove padding
     $str =~ tr|A-Za-z0-9+/| -_|;            # convert to uuencoded format
-    while ($str =~ /(.{1,60})/gs) {
-	my $len = chr(32 + length($1)*3/4); # compute length byte
-	$res .= unpack("u", $len . $1 );    # uudecode
+
+    ## I guess this could be written as
+    #return unpack("u", join('', map( chr(32 + length($_)*3/4) . $_,
+    #			$str =~ /(.{1,60})/gs) ) );
+    ## but I do not like that...
+    my $uustr = '';
+    my ($i, $l);
+    $l = length($str) - 60;
+    for ($i = 0; $i <= $l; $i += 60) {
+	$uustr .= "M" . substr($str, $i, 60);
     }
-    $res;
+    $str = substr($str, $i);
+    # and any leftover chars
+    if ($str ne "") {
+	$uustr .= chr(32 + length($str)*3/4) . $str;
+    }
+    return unpack ("u", $uustr);
 }
 
 # Set up aliases so that these functions also can be called as
